package com.goats.pojo;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;

import java.io.IOException;
import java.util.Arrays;

/**
 *  缓存数据结构，用于存储每个设备的数据包
 */
public class DataPacket {
    private String deviceId; // 设备ID
    private String[] dataParts = new String[3];  // 用来存储三部分数据
    private boolean[] receivedSeqs = new boolean[3];  // 记录哪些包已经收到
    private long lastUpdateTime;  // 最后一次更新时间

    public DataPacket(String deviceId, Long timestamp, int seq, String data) {
        this.deviceId = deviceId;
        addPacket(seq, data);
        this.lastUpdateTime = System.currentTimeMillis();
    }

    // 添加数据包
    public void addPacket(int seq, String data) {
        if (seq >= 1 && seq <= 3) {
            this.dataParts[seq - 1] = data;
            this.receivedSeqs[seq - 1] = true;
            this.lastUpdateTime = System.currentTimeMillis();
        }
    }

    // 判断是否已经接收到所有数据包
    public boolean isComplete() {
        for (boolean received : receivedSeqs) {
            if (!received) {
                return false;
            }
        }
        return true;
    }

    // 判断是否已经接收到某个包
    public boolean isSeqReceived(int seq) {
        return seq >= 1 && seq <= 3 && receivedSeqs[seq - 1];
    }

    /**
     * 获取合并后的数据
     * @return 合并后的 JSON 字符串
     * @throws IOException 如果 JSON 解析失败
     */
    public String getMergedData() throws IOException {
        // 调用 mergeJsonStrings 方法合并 JSON 数据
        return mergeJsonStrings(dataParts[0], dataParts[1], dataParts[2]);
    }

    // 获取最后一次更新时间
    public long getLastUpdateTime() {
        return lastUpdateTime;
    }

    /**
     * 合并多个 JSON 字符串
     * @param jsonStrings 多个 JSON 字符串
     * @return 合并后的 JSON 字符串
     * @throws IOException 如果 JSON 解析失败
     */
    public static String mergeJsonStrings(String... jsonStrings) throws IOException {
        // 创建 ObjectMapper 实例
        ObjectMapper mapper = new ObjectMapper();

        // 创建一个空的 ObjectNode 来存放合并后的内容
        ObjectNode mergedNode = mapper.createObjectNode();

        // 遍历传入的所有 JSON 字符串
        for (String jsonString : jsonStrings) {
            // 解析当前 JSON 字符串为 JsonNode
            JsonNode jsonNode = mapper.readTree(jsonString);

            // 合并字段
            jsonNode.fieldNames().forEachRemaining(fieldName -> {
                // 忽略重复的字段 seq, Date 和 DeviceID，只保留一个
                if (!fieldName.equals("seq") && !fieldName.equals("Date")) {
                    mergedNode.set(fieldName, jsonNode.get(fieldName));
                }
            });
        }

        // 返回合并后的 JSON 字符串
        return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(mergedNode);
    }

    public String getDeviceId() {
        return deviceId;
    }

    public void setDeviceId(String deviceId) {
        this.deviceId = deviceId;
    }

    @Override
    public String toString() {
        return "DataPacket{" +
                "deviceId='" + deviceId + '\'' +
                ", dataParts=" + Arrays.toString(dataParts) +
                ", receivedSeqs=" + Arrays.toString(receivedSeqs) +
                '}';
    }
}
